<?php

use S2lowLegacy\Lib\OpenStackContainerFetcher;
use S2lowLegacy\Lib\OpenStackContainerWrapper;
use S2lowLegacy\Lib\OpenStackStateManager;
use Monolog\Logger;
use OpenStack\Identity\v3\Models\Token;
use OpenStack\ObjectStore\v1\Models\Container;
use OpenStack\ObjectStore\v1\Models\StorageObject;
use PHPUnit\Framework\MockObject\MockObject;
use Psr\Http\Message\StreamInterface;
use Psr\Log\NullLogger;

class OpenStackContainerWrapperTest extends PHPUnit\Framework\TestCase
{
    /**
     * @var MockObject
     */
    private $mockManager;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $this->mockManager = $this->getMockBuilder(OpenStackStateManager::class)
            ->disableOriginalConstructor()
            ->getMock();
    }

    private function getTokenMock(bool $isExpired = false)
    {
        $tokenMock = $this->getMockBuilder(Token::class)
            ->disableOriginalConstructor()
            ->getMock();

        $tokenMock->expects($this->any())
            ->method("hasExpired")
            ->willReturn($isExpired);

        return $tokenMock;
    }

    private function getContainerMock()
    {
        return $this->getMockBuilder(Container::class)
            ->disableOriginalConstructor()
            ->getMock();
    }

    private function getOpenStackContainerFetcherMock($expects, $willReturn)
    {
        /** @var  $openStackContainerFetcherMock OpenStackContainerFetcher | MockObject */
        $openStackContainerFetcherMock = $this->getMockBuilder(OpenStackContainerFetcher::class)
            ->disableOriginalConstructor()
            ->getMock();

        $openStackContainerFetcherMock->expects($expects)
            ->method('getNewTokenAndContainer')
            ->will($willReturn);

        return $openStackContainerFetcherMock;
    }

    public function testExecuteCreateObject()
    {
        $method = 'createObject';
        $willReturn = $this->getMockBuilder(StorageObject::class)
            ->disableOriginalConstructor()
            ->getMock();
        $options = ["options"];

        $methodCalled = $method;

        $containerMock = $this->getContainerMock();

        $containerMock->expects($this->once())
            ->method($methodCalled)
            ->willReturn($willReturn);

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->once(),
            $this->returnValue([$this->getTokenMock(),$containerMock])
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new Logger("test"),
            $this->mockManager
        );

        $this->assertEquals(
            $openStackContainerWrapper->$method($options),
            $willReturn
        );
    }

    /**
     * @throws Exception
     */

    public function testExecuteDownload()
    {
        /** @var $streamInterfaceMock StreamInterface  */
        $streamInterfaceMock = $this->getMockBuilder(StreamInterface::class)
        ->disableOriginalConstructor()
        ->getMock();

        $StorageObjetMock = $this->getMockBuilder(StorageObject::class)
            ->disableOriginalConstructor()
            ->getMock();

        $StorageObjetMock->expects($this->once())
            ->method('download')
            ->willReturn($streamInterfaceMock);

        $containerMock = $this->getContainerMock();

        $containerMock->expects($this->once())
            ->method('getObject')
            ->willReturn($StorageObjetMock);

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->once(),
            $this->returnValue([$this->getTokenMock(),$containerMock])
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new NullLogger(),
            $this->mockManager
        );

        $this->assertEquals(
            $openStackContainerWrapper->download("options"),
            $streamInterfaceMock
        );
    }

    /**
     * @throws Exception
     */

    public function testExecuteObjectExists()
    {
        $willReturn = true;
        $options = "options";

        $containerMock = $this->getContainerMock();

        $containerMock->expects($this->once())
            ->method('objectExists')
            ->with($this->equalTo($options))
            ->willReturn($willReturn);

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->once(),
            $this->returnValue([$this->getTokenMock(),$containerMock])
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new NullLogger(),
            $this->mockManager
        );

        $this->assertEquals(
            $openStackContainerWrapper->objectExists($options),
            $willReturn
        );
    }

    /**
     * @throws Exception
     */

    public function testExecuteDelete()
    {
        $options = "options";

        $storageObjectNameMock = $this->getMockBuilder(StorageObject::class)
            ->disableOriginalConstructor()
            ->getMock();

        $storageObjectNameMock->expects($this->any())->method('delete')->willReturn(true);

        $containerMock = $this->getContainerMock();

        $containerMock->expects($this->once())
            ->method('getObject')
            ->with($this->equalTo($options))
            ->willReturn($storageObjectNameMock);

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->once(),
            $this->returnValue([$this->getTokenMock(),$containerMock])
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new NullLogger(),
            $this->mockManager
        );

        $this->assertEquals(
            true,
            $openStackContainerWrapper->delete($options)
        );
    }

    /**
     * @throws Exception
     */

    public function testExecuteFunctionOnContainerWithOutdatedToken()
    {

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->exactly(2),
            $this->onConsecutiveCalls(
                [$this->getTokenMock(true),$this->getContainerMock()],
                [$this->getTokenMock(false),$this->getContainerMock()]
            )
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new NullLogger(),
            $this->mockManager
        );

        // Première connexion : le token est périmé mais le getNewTokenAndContainer renvoie
        // quand même un container
        $openStackContainerWrapper->createObject(["options"]);
        // Deuxième connexion : le token périmé est détecté. Nouvelle appel à
        // openStackContainerFetcher
        // Première éxécution de createObject sur la deuxième instance de
        // $containerMock
        $openStackContainerWrapper->createObject(["options"]);
        //Troisième connexion : le token est ok. Aucun appel à openStackContainerFetcher
        // Deuxième éxécution de createObject sur la deuxième instance de
        // $containerMock
        $openStackContainerWrapper->createObject(["options"]);
    }

    /**
     * @throws Exception
     */

    public function testResetConnexion()
    {

        $firstContainerMock = $this->getContainerMock();

        $firstContainerMock->expects($this->once())
            ->method('createObject')
            ->with(["options1"]);

        $secondContainerMock = $this->getContainerMock();

        $secondContainerMock->expects($this->exactly(2))
            ->method('createObject')
            ->with(["options2"]);

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->exactly(2),
            $this->onConsecutiveCalls(
                [$this->getTokenMock(),$firstContainerMock],
                [$this->getTokenMock(),$secondContainerMock]
            )
        );

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            new NullLogger(),
            $this->mockManager
        );

        // Première connexion : premier appel à identityV3 et objectStoreV1
        $openStackContainerWrapper->createObject(["options1"]);
        $openStackContainerWrapper->resetConnection();
        // Deuxième connexion : deuxième appel à identityV3 et objectStoreV1
        // Première éxécution de createObject sur la deuxième instance de
        // $containerMock
        $openStackContainerWrapper->createObject(["options2"]);
        //Troisième connexion : le token en ok. Aucun appel à identityV3
        // et objectStoreV1
        // Deuxième éxécution de createObject sur la deuxième instance de
        // $containerMock
        $openStackContainerWrapper->createObject(["options2"]);
    }

    /**
     * @throws Exception
     */

    public function testExecuteFailsOnce()
    {

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->exactly(1),
            $this->onConsecutiveCalls($this->throwException(new BadMethodCallException("Exception1")))
        );

        /** @var Logger|MockObject $logger */
        $logger = $this->getMockBuilder(Logger::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->mockManager->expects($this->once())->method('declareException');

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            $logger,
            $this->mockManager
        );

        $openStackContainerWrapper->objectExists("ObjetTest");
    }

    /**
     * @throws Exception
     */

    public function testExecuteWithResetNeeded()
    {

        $containerMock = $this->getMockBuilder(Container::class)
            ->disableOriginalConstructor()
            ->getMock();

        $openStackContainerFetcherMock = $this->getOpenStackContainerFetcherMock(
            $this->exactly(1),
            $this->returnValue(["token",$containerMock])
        );

        /** @var Logger|MockObject $logger */
        $logger = $this->getMockBuilder(Logger::class)
            ->disableOriginalConstructor()
            ->getMock();

        $openStackContainerWrapper = new OpenStackContainerWrapper(
            $openStackContainerFetcherMock,
            $logger,
            $this->mockManager
        );

        $openStackContainerWrapper->objectExists("ObjetTest");
    }
}
